#[macro_export]
macro_rules! get_widget {
    ($builder:expr, $wtype:ty, @$name:ident) => {{
        let $name: $wtype = $builder
            .object(stringify!($name))
            .expect(&format!("Could not find widget \"{}\" of type \"{}\"", stringify!($name), stringify!($wtype)));
        $name
    }};
    ($builder:expr, $wtype:ty, $name:ident) => {
        let $name: $wtype = $builder
            .object(stringify!($name))
            .expect(&format!("Could not find widget \"{}\" of type \"{}\"", stringify!($name), stringify!($wtype)));
    };
}

#[macro_export]
macro_rules! spawn {
    ($future:expr) => {
        let ctx = glib::MainContext::default();
        ctx.spawn_local($future);
    };
}

#[macro_export]
macro_rules! send {
    ($sender:expr, $action:expr) => {
        if let Err(err) = $sender.send($action) {
            error!("Failed to send \"{}\" action due to {}", stringify!($action), err);
        }
    };
}

#[macro_export]
macro_rules! action {
    ($actions_group:expr, $name:expr, $callback:expr) => {
        let simple_action = gio::SimpleAction::new($name, None);
        simple_action.connect_activate($callback);
        $actions_group.add_action(&simple_action);
    };
    ($actions_group:expr, $name:expr, $param_type:expr, $callback:expr) => {
        let simple_action = gio::SimpleAction::new($name, $param_type);
        simple_action.connect_activate($callback);
        $actions_group.add_action(&simple_action);
    };
}

#[macro_export]
macro_rules! stateful_action {
    ($actions_group:expr, $name:expr, $state:expr, $callback:expr) => {
        let simple_action = gio::SimpleAction::new_stateful($name, None, &$state.to_variant());
        simple_action.connect_activate($callback);
        $actions_group.add_action(&simple_action);
    };
    ($actions_group:expr, $name:expr, $param_type:expr, $state:expr, $callback:expr) => {
        let simple_action = gio::SimpleAction::new_stateful($name, $param_type, &$state.to_variant());
        simple_action.connect_activate($callback);
        $actions_group.add_action(&simple_action);
    };
}

#[macro_export]
macro_rules! get_action {
    ($actions:expr, @$name:ident) => {{
        let $name = $actions
            .lookup_action(stringify!($name))
            .expect(&format!("Could not find action \"{}\"", stringify!($name)))
            .downcast::<gio::SimpleAction>()
            .unwrap();
        $name
    }};

    ($actions:expr, $name:ident) => {
        let $name = $actions
            .lookup_action(stringify!($name))
            .expect(&format!("Could not find action \"{}\"", stringify!($name)))
            .downcast::<gio::SimpleAction>()
            .unwrap();
    };
}

#[macro_export]
macro_rules! add_style_class {
    ($widget:expr, @$name:ident) => {{
        let ctx = $widget.style_context();
        ctx.add_class(stringify!($name));
    }};
    ($widget:expr, $names:expr) => {{
        let ctx = $widget.style_context();
        for name in $names {
            ctx.add_class(name);
        }
    }};
}
